import math
import unittest

from ..linexprs import Symbol
from ..coordinates import *


class TestPoint(unittest.TestCase):

    def setUp(self):
        self.x = Symbol('x')
        self.y = Symbol('y')
        self.z = Symbol('z')
        self.pt1 = Point({self.x: 10, self.y: 5, self.z: 1})
        self.pt2 = Point({self.x: 15, self.y: 40, self.z: 60})
        self.vec1 = Vector({self.x: 20, self.y: 30, self.z: 40})

    def test_add(self):
        self.assertEqual(self.pt1 + self.vec1, Point({self.x: 30, self.y: 35, self.z: 41}))
        with self.assertRaises(TypeError):
            self.pt1 + self.pt2

    def test_eq(self):
        self.assertEqual(self.pt1, self.pt1)
        self.assertNotEqual(self.pt1, self.pt2)
        self.assertNotEqual(self.pt1, self.vec1)

    def test_sub(self):
        self.assertEqual(self.pt1 - self.pt2, Vector({self.x: -5, self.y: -35, self.z: -59}))
        self.assertEqual(self.pt1 - self.vec1, Point({self.x: -10, self.y: -25, self.z: -39}))


class TestVector(unittest.TestCase):

    def setUp(self):
        self.x = Symbol('x')
        self.y = Symbol('y')
        self.z = Symbol('z')
        self.pt1 = Point({self.x: 10, self.y: 5, self.z: 1})
        self.pt2 = Point({self.x: 15, self.y: 40, self.z: 60})
        self.vec1 = Vector({self.x: 20, self.y: 30, self.z: 40})
        self.vec2 = Vector({self.x: 45, self.y: 70, self.z: 80})

    def test_add(self):
        self.assertEqual(self.vec1 + self.pt1, Point({self.x: 30, self.y: 35, self.z: 41}))
        self.assertEqual(self.vec1 + self.vec2, Vector({self.x: 65, self.y: 100, self.z: 120}))

    def test_angle(self):
        self.assertEqual(math.degrees(self.vec1.angle(self.vec1)), 0)
        self.assertAlmostEqual(math.degrees(self.vec1.angle(self.vec2)), 4.15129, places=5)
        self.assertAlmostEqual(math.degrees(self.vec2.angle(self.vec1)), 4.15129, places=5)

    def test_cross(self):
        self.assertEqual(self.vec1.cross(self.vec2), Vector({self.x: -400, self.y: 200, self.z: 50}))

    def test_div(self):
        self.assertEqual(self.vec1 / 10, Vector({self.x: 2, self.y: 3, self.z: 4}))

    def test_dot(self):
        self.assertEqual(self.vec1.dot(self.vec2), 6200)

    def test_eq(self):
        self.assertEqual(self.vec1, self.vec1)
        self.assertNotEqual(self.vec1, self.vec2)

    def test_mul(self):
        self.assertEqual(75 * self.vec1, Vector({self.x: 1500, self.y: 2250, self.z: 3000}))
        self.assertEqual(self.vec1 * 75, Vector({self.x: 1500, self.y: 2250, self.z: 3000}))

    def test_neg(self):
        self.assertEqual(-self.vec1, Vector({self.x: -20, self.y: -30, self.z: -40}))

    def test_norm(self):
        self.assertAlmostEqual(self.vec1.norm(), 53.85165, places=5)

    def test_norm2(self):
        self.assertEqual(self.vec1.norm2(), 2900)

    def test_asunit(self):
        unit = self.vec1.asunit()
        self.assertAlmostEqual(unit[self.x], 0.37139, 5)
        self.assertAlmostEqual(unit[self.y], 0.55709, 5)
        self.assertAlmostEqual(unit[self.z], 0.74278, 5)

    def test_sub(self):
        self.assertEqual(self.vec1 - self.pt1, Point({self.x: 10, self.y: 25, self.z: 39}))
        self.assertEqual(self.vec1 - self.vec2, Vector({self.x: -25, self.y: -40, self.z: -40}))
